@@ -211,6 +211,10 @@ struct _MetaWindow
 
   /* Transient parent is a root window */
   guint transient_parent_is_root_window : 1;
+
+  /* Info on which props we got our attributes from */
+  guint using_net_wm_name : 1; /* vs. plain wm_name */
+  guint using_net_wm_icon_name : 1; /* vs. plain wm_icon_name */
   
   /* Number of UnmapNotify that are caused by us, if
    * we get UnmapNotify with none pending then the client
